! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!**************************************************************
! January 2002         
! Compares OM photometry with WFU calibration data%expI
!***************************************************************
module astrometry_mod

    
   use param
   use cal
   use dal
   use sla

   use errorHandling  
   use omallocation_mod
  
   use omgraph_mod  
   use omnumerics_mod
   use statistics_mod
   use omutility_mod
   use leastsquares_mod
   implicit none
  
type AstrometryData
   
    real(single) :: raOffset, raOffsetErr
    real(single) :: decOffset, decOffsetErr
    real(single) :: rotation, rotationError
    real(single) :: rmsResidual
    integer :: noMatches
    real(double) :: JD
  
    real(single) :: raPoint, decPoint
end type AstrometryData
 


  
contains
!---------------------------------------------------------------------
subroutine astrometry
!-------------------------------------------------------------------
    type(AstrometryData), pointer :: data(:) => null() 
    integer i, n
    real(single) :: xmin, xmax, ymin, ymax
    real(single), pointer, dimension(:) :: xData=>null(), dxData=>null()
    real(single), pointer, dimension(:) :: yData=>null(), dyData=>null()
    real(single) ::  mean1, standardDeviation1
    real(single) ::  mean2, standardDeviation2
    real(single) :: jd0, binWidth
    character(200) :: title, xLabel, yLabel, text1

    call loadData(data)

    jd0 = minval(data%JD)
    data%JD = data%JD - jd0


    call pgbegin(0, "astrometry.ps/VCPS", 1, 1)
   
  

    n = size(data)
    xdata => allocateSingle(1, n, "xdata")
    dxdata => allocateSingle(1, n, "dxdata")
    ydata => allocateSingle(1, n, "ydata")
    dydata => allocateSingle(1, n, "dydata")

    xmin = -7.0
    xmax = 7.0
    ymin = -7.0
    ymax = 7.0

    title = "OM CATALOGUE ASTROMETRY"
    call DrawGraph(n, data%raOffset,  data%raOffsetErr, &
                      data%decOffset,  data%decOffsetErr, &
                  trim(title), "RA OFFSET (arcsecs)", "DEC OFFSET (arcsecs)", &
                  0.1, 0.8, 0.1, 0.8, xMin, xMax, xMin, xMax, .true.)


    xdata = data%raOffset
    dxdata = data%raOffsetErr
    ydata = data%decOffset
    dydata = data%decOffsetErr

    
!    call ComputeAverage(xData, dxData, nSize, mean1, standardDeviation1)
    call ComputeMean(xData, dxData, n, mean1, standardDeviation1)
    call ComputeMean(yData, dyData, n, mean2, standardDeviation2)
 

    write(text1, 100) "Mean delta RA offset=", mean1, " standard deviation=", standardDeviation1, " (n=", n, ")"
100 format(a, f8.2, a, f5.2, a, i5, a)
    call pgsch(0.6)
    call pgslw(1)
    call pgmtxt("T", 1.5, 0.0, 0.0, trim(text1))
    write(text1, 100) "Mean delta dec offset=", mean2, " standard deviation=", standardDeviation2, " (n=", n, ")"
    call pgsch(0.6)
    call pgslw(1)
    call pgmtxt("T", 0.5, 0.0, 0.0, trim(text1))
    call pgsls(2)
    call pgmove(0.0, ymin)
    call pgdraw(0.0, ymax)
    call pgmove(xmin, 0.0)
    call pgdraw(xmax, 0.0)


    call pgsch(0.6)
    call pgslw(1)


    call pgsfs(2)

    call pgsls(1)



    !*******************************************************
    !
    !******************************************************
    do i = 1, 5
        call pgsci(i)
        call pgcirc(0.0, 0.0, real(i))

        write(text1, '(i1, a)') i, " arcsec radius"
        call pgmtxt("T", -real(i), 0.0, 0.0, trim(text1))


    end do

    call pgsci(1)









    call pgpage
    !*******************************************************************
    ! Next graph
    !******************************************************************


 

    
    
    dydata = 0.0
    xmin = int(jd0)
    xmax = ceiling(maxval(data%jd))

    write(xLabel, '(a, f10.4)') "Modified Julian Day - ", JD0
    call DrawGraph(n, real(data%JD),  dyData, &
                      data%raOffset,  data%raOffsetErr, &
                 
                  trim(title), "", "RA OFFSET (arc secs)", &
                  0.1, 0.9, 0.525, 0.9)








    !*******************************************************************
    ! Next graph
    !******************************************************************
   
    dydata = 0.0
    call DrawGraph(n, real(data%JD),  dyData, &
                      data%decOffset,  data%decOffsetErr, &            
                  "", trim(xlabel), "DEC OFFSET (arc secs)", &
                  0.1, 0.9, 0.1, 0.475)













    !*******************************************************************
    ! Next graph
    !******************************************************************
    call pgpage
   
    binwidth = 0.25

    call Histogram(n, data%raOffset, binwidth,  &            
                  "RA OFFSET (arc secs)", trim(title), &
                  0.1, 0.9, 0.1, 0.475, xmin=0.0, xmax=5.0)
    call pgpage

    
    !*******************************************************************
    ! Next graph
    !******************************************************************
    call pgpage
   
    binwidth = 0.25

    call Histogram(n, data%decOffset, binwidth,  &            
                  "DEC OFFSET (arc secs)", trim(title), &
                  0.1, 0.9, 0.1, 0.475, xmin=0.0, xmax=5.0)
    call pgpage

    





    !*******************************************************************
    ! Next graph
    !******************************************************************
    call pgpage
   
    binwidth = 0.1

    call Histogram(n, data%rmsresidual, binwidth,  &            
                  "RMS RESIDUAL (arcsecs)", trim(title), &
                  0.1, 0.9, 0.1, 0.6, xmin=0.0, xmax=2.0)
    dxdata = 1.0
    call ComputeMean( data%rmsResidual, dxdata, n, mean1, standardDeviation1)
 

    write(text1, 100) "Mean RMS residual=", mean1, " standard deviation=", standardDeviation1, " (n=", n, ")"

    call pgsch(0.7)
    call pgslw(1)
    call pgmtxt("T", -1.0, 0.0, 0.0, trim(text1))






    call pgclos
    stop






    !*******************************************************************
    ! RA OM Pointing direction
    !******************************************************************
    call pgpage
   
    binwidth = 20.0

    title = "OM CATALOGUE OM POINTING DIRECTIONS"


    call Histogram(n, data%raPoint, binwidth,  &            
                  "OM RA POINTING DIRECTION (degrees)", trim(title), &
                  0.1, 0.9, 0.1, 0.475, xmin=0.0, xmax=360.0)
    call pgpage

    
    !*******************************************************************
    ! DEC OM Pointing direction
    !******************************************************************
    call pgpage
   
    binwidth = 10.0

    call Histogram(n, data%decPoint, binwidth,  &            
                  "OM DEC POINTING DIRECTION (degrees)", trim(title), &
                  0.1, 0.9, 0.1, 0.475, xmin=-90.0, xmax=90.0)
 


    call pgpage

    
 

    !*******************************************************************
    ! Next graph
    !******************************************************************
    call pgpage
    title = "OM CATALOGUE ASTROMETRY (30 DAY BINNING)"
 
    call Bindata(data, 30.0)
  
    n = size(data)

    if (n .gt. 10) then
        dydata = 0.0
        xmin = int(jd0)
        xmax = ceiling(maxval(data%jd))

        write(xLabel, '(a, f10.4)') "Modified Julian Day - ", JD0
        call DrawGraph(n, real(data%JD),  dyData, &
                      data%raOffset,  data%raOffsetErr, &
                 
                  trim(title), trim(xlabel), "RA OFFSET (arc secs)", &
                  0.1, 0.9, 0.525, 0.9,  ymin1=-10.0, ymax1=10.0)
        call pgslw(1)
        call pgline(n, data%JD, data%RAOffset)
        call LeastSquaresFit10(real(data%JD), data%raOffset, data%raoffsetErr)

        !*******************************************************************
        ! Next graph
        !******************************************************************
        
   
        dydata = 0.0
        call DrawGraph(n, real(data%JD),  dyData, &
                      data%decOffset,  data%decOffsetErr, &            
                  "", trim(xlabel), "DEC OFFSET (arc secs)", &
                  0.1, 0.9, 0.1, 0.475, ymin1=-10.0, ymax1=10.0)

        call pgslw(1)
        call pgline(n, data%JD, data%DecOffset)
        call LeastSquaresFit10(real(data%JD), data%DecOffset, data%DecoffsetErr)
    end if

    call pgclos
  
    if (associated(Data)) deallocate(data)
    deallocate(xData, dxdata, yData, dyData)
end subroutine astrometry
!************************************************************************
!
!
!***********************************************************************

subroutine loadData(data)
    type(AstrometryData), pointer :: data(:)
    type(AstrometryData), pointer :: tempData(:)

    character(200) :: fileName
    integer :: i, n, count
    real :: x
    real(single), pointer :: sortdata(:) => null()
    integer(int32), pointer :: index(:) => null()
    type(dataSetT) :: dataSetH
    type(tableT)   :: tableH


    fileName = trim(stringParameter("set"))
     
    dataSetH = dataset( fileName, read)
    tableH = table( dataSetH, "SUMMARY")

    count = numberOfRows( tableH )

    allocate(data(1 : count))

    call ReadReal32Column( tableH, "RA_PNT",   count, 0, data%rapoint )
    call ReadReal32Column( tableH, "DEC_PNT",  count, 0, data%decPoint)

    call ReadReal32Column( tableH, "RAOFFSET", count, 0, data%raOffset )
    call ReadReal32Column( tableH, "RAOFFERR", count, 0, data%raoffseterr)
    call ReadReal32Column( tableH, "DEOFFSET", count, 0, data%decoffset)
    call ReadReal32Column( tableH, "DEOFFERR", count, 0, data%decoffseterr)
    call ReadReal32Column( tableH, "RMSRESID", count, 0, data%rmsresidual)
    call ReadReal64Column( tableH, "MJD", count, 0, data%jd)

    n = 0
    do i = 1, count
        if (data(i)%raoffsetErr .gt. 0.0 .and. data(i)%decoffseterr .gt. 0.0) then
            n = n + 1
            data(n) = data(i)
        end if
    end do
 
    count = n 
    call release( dataSetH )

    ! Sort the data into increasing time order
    sortData => allocateSingle(1, count, "sortdata")
    index => allocateInt32(1, count, "index")

    sortData = data(1 : count)%JD
    call IndexSort(sortData, index, count)
    allocate(tempData(1 : count))
    tempdata = data(1 : count)

    deallocate(data)
    allocate(data(1 : count))
    do i = 1, count
        data(i) = tempData(index(i))
       
    end do
    deallocate(sortdata, tempdata, index)
end subroutine loadData
!*******************************************************************
!
!
!*******************************************************************
subroutine ComputeMean(xData, dxData, n, mean, standardDeviation)
    
    real(single), intent(in) :: xData(:), dxData(:)
    integer, intent(in) :: n
    real(single), intent(out) :: mean, standardDeviation
    real(double) :: sum1, sum2, sum3, weight, difference, d
    integer :: i

    sum1 = 0.0d0
    sum2 = 0.0d0
    sum3 = 0.0d0

    do i = 1, n
        weight = 1.0d0 / (dxData(i) * dxData(i))
        sum1 = sum1 + xData(i) * weight
        sum2 = sum2 + weight
        sum3 = sum3 + weight * weight
    end do

    mean = sum1 / sum2

    sum1 = 0.0d0 

    do i = 1, n
        difference = xdata(i) - mean       
        
        weight = 1.0_double / (dxData(i) * dxData(i))
        sum1 = sum1 + weight * difference * difference
         
    end do

    d = sum2 - sum3 / sum2

    standardDeviation = dsqrt(sum1 / d) ! The standard deviatio

end subroutine ComputeMean
!********************************************************************
!
!
!********************************************************************

subroutine Bindata(data, binTime)
    type(AstrometryData), pointer :: data(:)
    real(single), intent(in) :: binTime
    type(AstrometryData), pointer :: tempData(:) => null()
 
    integer :: i, j, k, l, n, jStart
    real(single) :: startTime, finishTime
    real(single) :: mean, standardDeviation
   

    n = size(data)
    k = 0
    startTime = data(1)%JD
    finishTime = data(n)%JD

    allocate(tempdata(1 : n))
    
    l = 1
    jStart = 2
    tempData(1) = data(1)

iloop :    do i = 1, n
     
        do j = jStart, n
        
            if ((data(j)%jd - startTime) .gt. binTime) then
                startTime = data(j)%JD
           
                k = k + 1
                ! Average the data
                
                call ComputeMean(tempData%raOffset, tempData%raOffsetErr, l, mean, standardDeviation)

                data(k)%raOffset = mean
                data(k)%raOffsetErr = standardDeviation
                call ComputeMean(tempData%decOffset, tempData%decOffsetErr, l, mean, standardDeviation)

                data(k)%decOffset = mean
                data(k)%decOffsetErr = standardDeviation

                  
                data(k)%Jd = (tempData(l)%JD + tempdata(1)%JD) / 2.0
              
                
                jStart = j 

                l = 0
                exit
            else
                l = l + 1
                
   

                tempdata(l) = data(j)     
                if (j .eq. n) exit iloop                  
            end if
        end do


        if (jStart .eq. n) exit
    end do iloop    


    if (l .gt. 0) then
        k = k + 1
        call ComputeMean(tempData%raOffset, tempData%raOffsetErr, l, mean, standardDeviation)

        data(k)%raOffset = mean
        data(k)%raOffsetErr = standardDeviation
        call ComputeMean(tempData%decOffset, tempData%decOffsetErr, l, mean, standardDeviation)

        data(k)%decOffset = mean
        data(k)%decOffsetErr = standardDeviation          
        data(k)%Jd = (tempData(l)%JD + startTime) / 2.0

    end if       
    tempData = data
    deallocate(data)
    allocate(data(1 : k))
    data = tempData(1 : k)
    deallocate(tempData)
    nullify(tempdata) 



end subroutine Bindata

subroutine LeastSquaresFit10(xData, ydata, dyData)
    
    real(single) :: xData(:), ydata(:), dyData(:)
    integer :: i, n, ifail, nCoeffs
    real(double) :: weight
    real(double) :: beta(1 : 4), x(1 : 4), var(1 : 20)
    real(double) :: d, sterr(5)
    character(200) :: text    
    
    n = size(xData)
    nCoeffs = 3
    call startup(nCoeffs, .true.)
    call tolset
    
    do i = 1, n
        if (dyData(i) .gt. 0.0) then
            weight = 1.0 / (dyData(i) * dyData(i))
        else
            weight = 1.0
        end if
!        xData(i) = real(i)
!        ydata(i) = 4.5 + 6.9 * xData(i)
!        weight = 1.0 
        x(1) = 1.0_double
        x(2) = dble(xdata(i)) / 1000.0
        x(3) = x(2) * x(2)
        x(4) = x(3) * x(2)
       
        call includ(weight, x, dble(ydata(i)))


    end do
   
    call regcf(beta, 2, ifail)
    call cov(2, d, var, 20, sterr, ifail)
    var = sqrt(var)
    write(*, *) "Computed regression coefficients (units of 1000 days)"

    do i = 1, 2
        write(*, *) i, ") ", beta(i), " +/-", var(i)
    end do
    write(text, '(a, f6.3, a, f6.3, a, f8.6, a, f8.6, a)')  "Regression line - y=(", beta(1), &
                                         "+/-", var(1), ") + (", beta(2), "+/-", var(2), ")*x"
    call pgsch(0.8)
    call pgmtxt("T", -1.0, 0.0, 0.0, trim(text))
    call pgmtxt("T", -2.0, 0.0, 0.0, "(Time (x) is in units of 1000 days)")

 
    call pgslw(1)
    call pgmove(xData(1), real(beta(1) + beta(2) * xdata(1) / 1000.0))
    call pgdraw(xData(n), real(beta(1) + beta(2) * xdata(n) / 1000.0))

end subroutine LeastSquaresFit10
end module astrometry_mod















