! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
 
module convert_mod
use dal
use sla
use cal
use getFilterID_mod

use omconstants_mod
public
contains

subroutine convert(x, y, ra0, dec0, ra, dec)
    real(double), intent(in) :: x, y, ra0, dec0
    real(double), intent(out) :: ra, dec


    call sla_dtp2s(x * degreesToRadians, y * degreesToRadians,  &
               ra0 * degreesToRadians, dec0 * degreesToRadians, &
               ra, dec)

!    ra = ra * radiansToDegrees
!    dec = dec * radiansToDegrees


end subroutine convert


!*********************************  
! Compute tangent plane coords
! xi_gs(i) & eta_gs(i)
!*********************************
subroutine computetangentplanecoords(ra0, dec0, ra, dec, x, y)
    real(double), intent(in) :: ra0, dec0, ra, dec
    real(double), intent(out) :: x, y
    integer :: j


    call sla_ds2tp( ra * degreesToRadians,  &
                    dec * degreesToRadians, &
                    ra0 * degreesToRadians, &
                    dec0 * degreesToRadians,&
                    x, y, j)


       

    !*********************************
    ! Check to see okay
    !********************************
    call SlaErrorCheck(j)
 
    x = x * radiansToDegrees
    y = y * radiansToDegrees
 

end subroutine computetangentplanecoords




!***********************************************************
!
!**********************************************************
real(double) Function getplatescale()


    integer :: filterIdInt


    !  Get the filter Id integer value
    filterIdInt = filterIdToInt("V")

    call CAL_setState(OM, filterId = filterIdInt)

 
    ! Get the Plate scale (arcsecs/pixel)
    call CAL_omGetPlateScale(0.55_double, getplateScale)
    




end function getplatescale
!**********************************
! subroutine SlaErrorCheck
!**********************************
subroutine SlaErrorCheck(j)
    integer, intent(in) :: j

    select case (j)
        case (1) 
            call error("toofar", "Guide star too far from axis")
        case (2)
            call error("antistar", "Anti-star on tangent plane")
        case (3)
            call error("toofar", "Anti-star too far from axis")
    end select
end subroutine SlaErrorCheck



 
end module convert_mod





