! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
module coordinates

use dal
use sla
use sasdal
use caloaldefs
use cal
use omconstants_mod
use omsourceparam_mod
use omdistortion_mod
use omutility_mod
use omsourceflags_mod
public

contains



subroutine initialise()


    !*********************
    ! Set the CAL state
    !*********************
    call CAL_setState( OM, ccdModeId=Image, filterId = filterIdToInt("V") )


    call initialiseDistortion(0, 0, 1, 1)


end subroutine initialise

subroutine addistortion(xin, yin, xout, yout)
    real(double), intent(in) :: xin, yin
    real(double), intent(out) :: xout, yout


    call addDistortion(xin, yin, xout, yout)


end subroutine addistortion




subroutine undistortion(xin, yin, xout, yout)
    real(double), intent(in) :: xin, yin
    real(double), intent(out) :: xout, yout


    call removeDistortion(xin, yin, xout, yout)


end subroutine undistortion


!*************************************************************************
!
!*************************************************************************
subroutine computestandardcoordinates(ra, dec, raPnt, decPnt, xi, eta)
    real(double), intent(in) :: ra, dec, raPnt, decPnt
    real(double), intent(out) :: xi, eta
    integer :: j
  


    !*********************************  
    ! Compute tangent plane coords
    ! xi_gs(i) & eta_gs(i)
    !*********************************
    j = 0
    

    call sla_ds2tp( ra * degreesToRadians, dec * degreesToRadians, &
         raPnt * degreesToRadians, decPnt * degreesToRadians, xi, eta, j)

    !*********************************
    ! Check to see okay
    !********************************
    call SlaErrorCheck(j)
    


end subroutine computestandardcoordinates





!**********************************
! subroutine SlaErrorCheck
!**********************************
subroutine SlaErrorCheck(j)
    integer, intent(in) :: j

    select case (j)
        case (1) 
            call error("toofar", "Guide star too far from axis")
        case (2)
            call error("antistar", "Anti-star on tangent plane")
        case (3)
            call error("toofar", "Anti-star too far from axis")
    end select
end subroutine SlaErrorCheck

!***************************************************************
!
!**************************************************************
real(double) function getplatescale()

    real(double) :: lambda = 0.55


    !*********************
    ! Set the CAL state
    !*********************
    call CAL_setState( OM, ccdModeId=IMAGE, filterId = filterIdToInt("V") )


    call CAL_omGetPlateScale( lambda, getplateScale )  ! Get the plate-scale (arcsecs per pixel) 


end function getplatescale

!***************************************************************
!
!**************************************************************
subroutine list(n, data)
    integer, intent(in) :: n
    real(double) :: data(1 : n)

    integer :: i
     
    do i = 1, n
        write(*, *) i, ") data=", data(i)
        data(i) = -data(i)
    end do


end subroutine list

!***************************************************************
!
!**************************************************************
subroutine list1(n, source)
    integer, intent(in) :: n
    type(sourceparameters) :: source(1 : n)

    integer :: i
     
    do i = 1, n
        write(*, *) i, ") x=", source(i)%xpos
        source(i)%xpos = -source(i)%xpos

    end do


end subroutine list1
!**********************************************************************
!
!**********************************************************************
subroutine windowparameters(fileName, x0, y0, binx, biny)
    !character(*), intent(in) :: fileName
    character(256), intent(in) :: fileName
    integer, intent(out) ::  x0, y0, binx, biny

    type(dataSetT) :: dataSetH
    integer :: i

    integer :: length1
    length1=len(fileName)
    write(*,*) "L172 length1=", length1
    ! Fine the index of the . before the file extension name
    i = index(fileName, ".")
    write(*,*) "L175 fileName=", fileName, " i=", i
    dataSetH = dataSet(fileName(1 : i + 3), read)
    call GetWindowParameters( dataSetH, x0, y0, binX, binY)
    
    call release(dataSetH)
    write(*, *) "L180 windowparameters x0=", x0, " y0=", y0, " binx=", binx, " biny=", biny


end subroutine windowparameters

!**********************************************************************
!
!**********************************************************************
!subroutine windowparameters1(fileName, stringLength, x0, y0, binx, biny)
!    !character(*), intent(in) :: fileName
!    character(256), intent(in) :: fileName
!    integer, intent(in) :: stringLength
!    integer, intent(out) ::  x0, y0, binx, biny
!
!    type(dataSetT) :: dataSetH
!    integer :: i
!
!    integer :: length1
!    write(*,*) "L198 stringLength=", stringLength
!    length1=len(fileName)
!    write(*,*) "L200 length1=", length1
!    ! Fine the index of the . before the file extension name
!    i = index(fileName, ".")
!    write(*,*) "L203 fileName=", fileName, " i=", i
!    dataSetH = dataSet(fileName(1 : i + 3), read)
!    call GetWindowParameters( dataSetH, x0, y0, binX, binY)
!    
!    call release(dataSetH)
!    write(*, *) "L208 windowparameters x0=", x0, " y0=", y0, " binx=", binx, " biny=", biny
!
!
!end subroutine windowparameters1

!**********************************************************************
!
!**********************************************************************
subroutine windowparameters1(fileNameBytes, stringLength, x0, y0, binx, biny)
    !character(*), intent(in) :: fileName
    !integer(kind=1), intent(in)  :: fileNameBytes(:)
    integer(kind=1), intent(in)  :: fileNameBytes(256)
    !character(1) :: oneSymbol
    integer, intent(in) :: stringLength
    integer, intent(out) ::  x0, y0, binx, biny
    character(256) :: fileName
    !character(1024) :: fileName2;

    type(dataSetT) :: dataSetH
    integer :: i
    
    integer :: length1

    
    !write(*,*) "L227 stringLength=", stringLength

       
    write (fileName,'(256a)') (achar(fileNameBytes(i)),i=1,stringLength)

    fileName =trim(fileName)
    length1=len(fileName)

    !write(*,*) "L251 length1=", length1
    !write(*,*) "L252 fileName=",trim(fileName)
    !write(*,*) "L256 trimmed length=", len(trim(adjustl(fileName)))

    i = index(fileName, ".", back=.true.)
    !write(*,*) "L259  i=", i
    dataSetH = dataSet(fileName(1 : i + 3), read)
    call GetWindowParameters( dataSetH, x0, y0, binX, binY)
    call release(dataSetH)
    !write(*, *) "L263 windowparameters x0=", x0, " y0=", y0, " binx=", binx, " biny=", biny


end subroutine windowparameters1

!**********************************************************************
!
!**********************************************************************
subroutine convertcoords(x, y, x0, y0, binx, biny)
    real(single), intent(inout) :: x, y
    integer, intent(in) ::  x0, y0, binx, biny

    call ConvertCoordsToFullFrame(x, y, x0, y0, binx, biny)



end subroutine convertcoords


end module coordinates
