! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!*********************************************************************************************************** 
module masterSrcTable_mod

    use dal
    use cal
    use param  
    use sasdal
    
    use OmAllocation_mod         ! Functions for allocating memory to arrays.
    use omconstants_mod
    use OmPhotometryDefs_mod
    use omutility_mod   
    use errorHandling

    implicit none 




 
  public CreateSourceList





  
    !**********************************************************************
    ! The following structure stores the parameters for a detected source
    !**********************************************************************

    type SourceParameters1
        integer(int32) :: number   = 0        ! The source number
        real(single)   :: xPos           = -1.0     ! x source pos (pixels)
        real(single)   :: yPos           = -1.0     ! y source pos (pixels)
        real(single)   :: rightAscension = -1.0     ! Right Ascension
        real(single)   :: declination    = -1.0     ! declination
        real(single)   :: poserr    = -1.0          ! error on position
        character(5)   :: filter
        real(single)   :: mjd 
        real(single)   :: exposure                  ! Exposure time (seconds)
        real(single)   :: significance      = nullValue     ! source significance
        real(single)   :: rawCountRate      = nullValue     ! raw count rate
        real(single)   :: rawCountRateErr   = nullValue     ! error on raw count rate
        real(single)   :: corrCountRate = nullValue     ! raw counts corrected for coincidence losses
        real(single)   :: corrCountRateErr = nullValue     ! error on source counts corrected for coincidence losses
        real(single)   :: magnitude
        real(single)   :: magnitudeErr
        real(single)   :: rawBackGroundCountRate  = nullValue! Source raw background counts
        real(single)   :: rawBackGroundCountRateErr  = nullValue ! Source raw background counts
        real(single)   :: corrBackgroundCountRate = nullValue ! corrected background counts
        real(single)   :: corrBackgroundCountRateErr = nullValue ! corrected background counts
        real(single)   :: psf1Correction = 0.0      ! psf correction to cal aperture
        real(single)   :: psf2Correction = 0.0      ! psf correction to cal aperture
        real(single)   :: coincidenceLossCorr = 0.0 ! Coincidence-loss correction
        real(single)   :: tdsCorr = nullValue       ! Time-dependent-sensitivity correction

        real(single)   :: semiA = nullValue         ! width of psf
        real(single)   :: semiAErr = nullValue            ! Error in width of psf

        real(single)   :: semiB = nullValue         ! width of psf
        real(single)   :: semiBErr = nullValue      ! Error in width of psf

        real(single)   :: theta  = nullValue        ! The position-angle for each source
        real(single)   :: thetaErr  = nullValue     ! The position-angle for each source

        real(single)   :: apertureRadius = -1.0     ! The semi-minor axis radius for each source
        real(single)   :: axialRatio = 1.0          ! The axial-ratio of aperture ellipse (1 = circle)
        real(single)   :: alpha = 0.0               ! The position-angle of the ellipse
        real(single)   :: innerBckRadius = -1.0     ! The semi-minor aperture radii for each source
        real(single)   :: outerBckRadius = -1.0     ! The semi-minor aperture radii for each source
        integer(int32) :: noSourcePixels = 0     ! The number of pixels included by a source
        integer(int8)  :: extensionFlag   = 0    ! Point-source (0), otherwise extended (1)
        integer(int8)  :: confusionFlag  = 0     !
        integer(int8)  :: qualityFlag  = 0       ! Set to 1 if 1 or more source pixels is bad
        integer(int16) :: iMin = 0               ! x - lower index image array bound of source
        integer(int16) :: iMax = 0               ! x - upper index image array bound of source
        integer(int16) :: jMin = 0               ! y - upper index image array bound of source
        integer(int16) :: jMax = 0               ! y - upper index image array bound of source
        character(100) :: comment
    end type SourceParameters1


interface convert
     module procedure convert1, convert2
end interface convert


contains

!***********************************************************************
!
!
!**********************************************************************
subroutine convert1( source1, source2 )
     type(sourceParameters) :: source1(:)
     type(sourceParameters1):: source2(:)
     integer i

     if (size(source1) .ne. size(source2)) then
          call fatal("masterscrtable different sizes")
     end if
     do i = 1, size(source1)

         source2(i)%number = source1(i)%number
         source2(i)%xpos = source1(i)%xpos
         source2(i)%ypos = source1(i)%ypos
         source2(i)%rightAscension = source1(i)%rightAscension
         source2(i)%declination = source1(i)%declination
         source2(i)%poserr = source1(i)%poserr
         source2(i)%exposure = source1(i)%exposure
         source2(i)%significance = source1(i)%significance
         source2(i)%rawCountRate = source1(i)%rawCountRate
         source2(i)%rawCountRateErr = source1(i)%rawCountRateErr
         source2(i)%corrCountRate = source1(i)%corrCountRate
         source2(i)%corrCountRateErr = source1(i)%corrCountRateErr
         source2(i)%magnitude = source1(i)%magnitude
         source2(i)%magnitudeErr = source1(i)%magnitudeERR
         source2(i)%rawBackgroundCountRate = source1(i)%rawBackgroundCountRate
         source2(i)%rawBackgroundCountRateErr = source1(i)%rawBackgroundCountRateErr
         source2(i)%corrBackgroundCountRate = source1(i)%corrBackgroundCountRate
         source2(i)%corrBackgroundCountRateErr = source1(i)%corrBackgroundCountRateErr
         source2(i)%psf1correction = source1(i)%psf1correction
         source2(i)%psf2correction = source1(i)%psf2correction
         source2(i)%tdscorr = source1(i)%tdscorr    
         source2(i)%semia = source1(i)%semia
         source2(i)%semiaErr = source1(i)%semiaErr
         source2(i)%semib = source1(i)%semib
         source2(i)%semibErr = source1(i)%semibErr
         source2(i)%theta = source1(i)%theta
         source2(i)%thetaErr = source1(i)%thetaErr
         source2(i)%apertureradius = source1(i)%apertureradius
         source2(i)%extensionFlag = source1(i)%extensionFlag
         source2(i)%qualityFlag = source1(i)%qualityFlag
         source2(i)%confusionFlag = source1(i)%confusionFlag
    
     end do
end subroutine convert1

!***********************************************************************
!
!
!**********************************************************************
subroutine convert2( source1, source2 )
     type(sourceParameters1) :: source1(:)
     type(sourceParameters)  :: source2(:)
     integer i

     if (size(source1) .ne. size(source2)) then
          call fatal("masterscrtable different sizes")
     end if
     do i = 1, size(source1)

         source2(i)%number = source1(i)%number
         source2(i)%xpos = source1(i)%xpos
         source2(i)%ypos = source1(i)%ypos
         source2(i)%rightAscension = source1(i)%rightAscension
         source2(i)%declination = source1(i)%declination
         source2(i)%poserr = source1(i)%poserr
         source2(i)%exposure = source1(i)%exposure
         source2(i)%significance = source1(i)%significance
         source2(i)%rawCountRate = source1(i)%rawCountRate
         source2(i)%rawCountRateErr = source1(i)%rawCountRateErr
         source2(i)%corrCountRate = source1(i)%corrCountRate
         source2(i)%corrCountRateErr = source1(i)%corrCountRateErr
         source2(i)%magnitude = source1(i)%magnitude
         source2(i)%magnitudeErr = source1(i)%magnitudeERR
         source2(i)%rawBackgroundCountRate = source1(i)%rawBackgroundCountRate
         source2(i)%rawBackgroundCountRateErr = source1(i)%rawBackgroundCountRateErr
         source2(i)%corrBackgroundCountRate = source1(i)%corrBackgroundCountRate
         source2(i)%corrBackgroundCountRateErr = source1(i)%corrBackgroundCountRateErr
         source2(i)%psf1correction = source1(i)%psf1correction
         source2(i)%psf2correction = source1(i)%psf2correction
         source2(i)%tdscorr = source1(i)%tdscorr    
         source2(i)%semia = source1(i)%semia
         source2(i)%semiaErr = source1(i)%semiaErr
         source2(i)%semib = source1(i)%semib
         source2(i)%semibErr = source1(i)%semibErr
         source2(i)%theta = source1(i)%theta
         source2(i)%thetaErr = source1(i)%thetaErr
         source2(i)%apertureradius = source1(i)%apertureradius
         source2(i)%extensionFlag = source1(i)%extensionFlag
         source2(i)%qualityFlag = source1(i)%qualityFlag
         source2(i)%confusionFlag = source1(i)%confusionFlag
    
     end do
end subroutine convert2



















!*********************************************
! Subroutine CreateSourceList
!*********************************************
subroutine CreateSourceList(dataSetH,   & 
                            nSources,   &
                            sourceData)
 

    type(DataSetT), intent(inout)  :: dataSetH
    integer(int32), intent(in) :: nSources 
    type(sourceParameters1), pointer :: sourceData(:)
    real(double) :: ra, dec   
 
      
    type(TableT) :: listTable
    type(ColumnT)        :: pcolumn                 ! pointer to a column

    integer (int32), pointer :: srcnum(:)  ! the source number
   
   
    integer(int32) :: i
  

    if (.not. associated(sourceData) .or. nSources .eq. 0) then
        return
    end if


    ! Write out the source list to the oswDataSet
    write(errstr,'(a)') "Creating source list"
    call message(AppMsg, Verbose)

  

    listTable = addTable(dataSetH, "SRCLIST1", nSources)

    !********************************
    ! now add the columns one by one
    ! starting with the source number
    !*********************************
    pcolumn = addColumn(listTable, "SRCNUM", Integer32, " ")
    srcnum => int32data(pcolumn)

    forall (i = 0 : nSources - 1) srcnum(i) = i + 1
   
    call release(pcolumn)


    call AddCharColumn(listTable, "FILTER", " ", "No of merged source", nsources, sourcedata%filter)

   
    ! Add the exposures
    call AddReal32Column(listTable, sourceData%exposure,  nSources, &
                         "EXPOSURE", "secs", "", nullValue)




    call AddInt32Column(listTable, sourceData%number,  nSources, &
                         "UNIQUE_SRCNUM", " ", "No of merged source")



  
    ! Add the source positions
    call AddReal32Column(listTable, sourceData%mjd,  nSources, &
                         "MJDREF", "pixels", " ", nullValue)


   
    ! Add the source positions
    call AddReal32Column(listTable, sourceData%xPos,  nSources, &
                         "XPOS", "pixels", "Centroided x-pixel position", nullValue)

    call AddReal32Column(listTable, sourceData%yPos,  nSources, &
                         "YPOS", "pixels", "Centroided y-pixel position", nullValue)


    call AddReal32Column(listTable, real(sourceData%rightAscension * degreesToRadians),  nSources, &
                         "RA", "deg", "Right Ascension", nullValue)

    call AddReal32Column(listTable, real(sourceData%declination * degreesToRadians),  nSources, &
                         "DEC", "deg", "Declination", nullValue)




    ! Galactic longitude and latitude


    do i = 1, nsources
        ra = dble( sourceData(i)%rightascension * degreestoRadians)
        dec = dble( sourceData(i)%declination * degreestoRadians)


        call sla_EQGAL( ra, dec, ra, dec )

        sourcedata(i)%rightascension = ra * radiansToDegrees
        sourcedata(i)%declination = dec * radiansToDegrees

    end do





    call AddReal32Column(listTable, sourceData%rightAscension,  nSources, &
                         "LII", "deg", "Right Ascension", nullValue)


    call AddReal32Column(listTable, sourceData%declination,  nSources, &
                         "BII", "deg", "Declination", nullValue)




 

  
    call AddReal32Column(listTable, sourceData%poserr,  nSources, &
                         "POSERR", "pixels", "Centroided pixel position error", nullValue)
     




     
    call AddReal32Column(listTable, sourceData%rawCountRate, nSources, &
                        "RATE", "counts/sec", "Source-background. No corrections have been applied", nullValue)

    call AddReal32Column(listTable, sourceData%rawCountRateErr, nSources, &
                        "RATE_ERR", "counts/sec", "No corrections have been applied", nullValue)
  

    call AddReal32Column(listTable, sourceData%rawBackGroundCountRate, nSources, &
              "BACKGROUND_RATE", "counts/sec",  "No corrections have been applied", nullValue)
    

    call AddReal32Column(listTable, sourceData%significance,  nSources, &
               "SIGNIFICANCE", "sigma", "(source-background)/sqrt(source+background)", nullValue)
 
    ! Add the coincidence-corrected source-count rates 
 
    call AddReal32Column(listTable,  sourceData%corrCountRate, nSources, &
                "CORR_RATE", "counts/sec", "Source-background. Corrected for coincidence losses, etc", nullValue)

    

    call AddReal32Column(listTable,  sourceData%corrCountRateErr, nSources, &
                "CORR_RATE_ERR", "counts/sec", "Corrected for coincidence losses", nullValue)
    

    
    call AddReal32Column(listTable, sourceData%corrBackGroundCountRate, nSources, &
                         "CORR_BACKGROUND_RATE", "counts/sec", "Corrected for coincidence losses, etc", nullValue)



    ! Add the coincidence-loss correction  
    call AddReal32Column(listTable, sourceData%psf1Correction, nSources, &
              "PSF1_CORR", "%", "PSF correction from mesurement to 12 pixel aperture", nullValue)


!    call AddReal32Column(listTable, sourceData%coincidenceLossCorr, nSources, &
!              "COINCLOSS_CORR", "%", "Coincidence-loss scaling factor", nullValue)


    call AddReal32Column(listTable, real(sourceData%rightAscension), nSources, &
              "SPB_COILOSS", " ", "(S+B coinc-loss corr)/(S+B un-coinc-loss corr)", nullValue)

    call AddReal32Column(listTable, real(sourceData%declination), nSources, &
              "BK_COILOSS", " ", "(B coinc-loss corr)/(B un-coinc-loss corr)", nullValue)




    ! Add the number of pixels associated with extended sources- for point-sources
    ! value set to NULL
    call AddInt32Column(listTable, sourceData%noSourcePixels, nSources, &
                "NOPIXELS", "pixels", "No of pixels assigned to extended sources")
 

    ! Add the aperture radius 
    call AddReal32Column(listTable, sourceData%apertureRadius, nSources, &
                    "APERTURE", "unbinned pixels", "Aperture radius used in photometry", nullValue)


    call AddReal32Column(listTable,  sourceData%semiA, nSources, &
                 "FWHM_MAJ", "arcsecs", "FWHM of semi-major axis", nullValue)
 
    call AddReal32Column(listTable,  sourceData%semiAErr, nSources, &
                 "FWHM_MAJ_ERR", "arcsecs", "FWHM error of semi-major axis", nullValue)
 
 

    call AddReal32Column(listTable,  sourceData%semiB, nSources, &
                 "FWHM_MIN", "arcsecs", "FWHM of semi-minor axis", nullValue)

    call AddReal32Column(listTable,  sourceData%semiBErr, nSources, &
                 "FWHM_MIN_ERR", "arcsecs", "FWHM error of semi-minor axis", nullValue)
 

 
    call AddReal32Column(listTable,  sourceData%theta, nSources, &
                 "PA", "degrees", "PA of semi-major axis (measured anti-clockwise from x-axis)", nullValue)
    call AddReal32Column(listTable,  sourceData%thetaErr, nSources, &
                 "PA_ERR", "degrees", "PA error of semi-major axis", nullValue)
 

    ! Add the quality flags
    call AddInt8Column(listTable,  sourceData%qualityFlag, nSources, &
                  "QFLAG", "none", "see comment below)")   ! Can be either 0 or 1
    call AddInt8Column(listTable,  sourceData%confusionFlag, nSources, &
                             "CFLAG", "none", "0=isolated")   ! 
    call AddInt8Column(listTable, sourceData%extensionFlag, nSources, &
              "EFLAG", "none", "0=point, 1=extended source")    ! Can be either 0 (point-source) or 1 (extended)
 
  
 
 
 
  
    call AddCharColumn(listTable, "SCREENING_COMMENT", " ", "Comment added during screening", nsources, sourcedata%comment)



    call addcomment(listTable, "QFLAG- bits set (colour refers to region colour)")
    call addcomment(listTable, "if QFLAG=0 then region colour will be either")
    call addcomment(listTable, "green (point-source) or cyan (extended source)")
    call addcomment(listTable, "0=Bad pixel (red)")
    call addcomment(listTable, "1=Source on possible read-out streak (magenta)")
    call addcomment(listTable, "2=Source on possible smoke-ring (yellow)")
    call addcomment(listTable, "3=Source on possible star diffraction-spike (white)")
    call addcomment(listTable, "4=Source likely affected by mod-8 pattern (black)")
    call addcomment(listTable, "5=Source likely near to a very bright source (black)")
    call addcomment(listTable, "6=Source within central enhanced region (red)")

    call addcomment(listTable, "CFLAG- bits set (colour refers to region colour")
    call addcomment(listTable, "0 (point-source only) one or more sources lie within a radius of 6 to 12 unbinned pixels")
    call addcomment(listTable, "1 (point-source only) one or more sources lie within a radius of 6 unbinned pixels")
    call addcomment(listTable, "2 Source lies near an image edge (blue)")
    call addcomment(listTable, "3 point-source lies partly, or entirely, within an extended-source region")
   

    !************************
    ! release the memory
    !************************
    call release(listTable)
  

end subroutine CreateSourceList





end module masterSrcTable_mod








