// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <fstream>
#include <iostream>
#include "omswsflags.h"
#include "Dal.h"
#include "param.h"
#include<cstring>
#include <cstdio>
#include "utility.h"
#include "sourcelist.h"
#include "qfile.h"
#include "qdir.h"
#include <cmath>
/******************************************************************
*
*******************************************************************/
F90MODPROCPROTO(void, coordinates, computestandardcoordinates)
       (double *ra, double *dec, double *raPnt, double *decPnt, double *xi, double *eta);



F90MODPROCPROTO(void, coordinates, list)
       (int *n, double *data);

F90MODPROCPROTO(void, coordinates, list1)
       (int *n, OmSourceParameter *data);


F90MODPROCPROTO(void, addflags, addsourceflags)
       (int *n, OmSourceParameter *data);


F90MODPROCPROTO(double, coordinates, getplatescale)();


F90MODPROCPROTO(void, addflags, createoutputlist)();

F90MODPROCPROTO(void, coordinates, initialise)();


F90MODPROCPROTO(void, coordinates, addistortion)(double *, double *, double *, double *);

F90MODPROCPROTO(void, coordinates, undistortion)(double *, double *, double *, double *);

//F90MODPROCPROTO(void, coordinates, windowparameters)(char[256], int *, int *, int *, int *);
F90MODPROCPROTO(void, coordinates, windowparameters1)(unsigned char *, size_t *, int *, int *, int *, int *);

F90MODPROCPROTO( void, coordinates, convertcoords)(float *x, float*y, int *x0, int *y0, int *binx, int *biny);


/*************************************************************************************
*
*************************************************************************************/



windowParameters getWindowParameters( string fileName );







void convertCoords( windowParameters &parameters, float &x, float &y);




void resetflags(  vector < OmSourceParameter > &source);







/**************************************************************************
*
***************************************************************************/
void setConfusionFlags( vector < OmSourceParameter > &source);


/**********************************************************************
*
**********************************************************************/
void omswsflags()
{


    /********************************************************
    * Create the output source-list
    ********************************************************/


    F90MODPROC( addflags, createoutputlist )();


    F90MODPROC( coordinates, initialise )();

    string swsFile = stringParameter( "set" );

  
   

    /***************************************************************
    * Load the data from the SWS source-list file
    ***************************************************************/


    int size = 0;
    vector < OmSourceParameter > source;
    string filter;
   
    if ( !loadSourceParameters( swsFile, "SRCLIST", size, source, filter))                     
    {

      cout << "bug somewhere- could not load file" <<swsFile << endl;
      Sas::fatal("Could not load file-", swsFile);


    }
   

    if(size == 0) return;

    /***************************************************
    * Reset the flags for this source list
    ****************************************************/
    //    resetflags(  source );






    /******************************************************************************
    * Convert x and y coordinates to full-frame coordinates
    *******************************************************************************/
    windowParameters parameter = getWindowParameters( swsFile );

    for(unsigned i = 0; i < source.size(); i++)
    {   


        convertCoords( parameter, source[i].xcoord, source[i].ycoord );

    }


    //    setConfusionFlags( source ); 

    int n = source.size();

    OmSourceParameter *source1 = new OmSourceParameter[n];
 

    for(int i = 0; i < n; i++)
    {
      source1[i] = source[i];
        source1[i].xcoord = source[i].xcoord; 
        source1[i].ycoord = source[i].ycoord; 

        source1[i].extensionflag = source[i].extensionflag;
        source1[i].confusionflag = source[i].confusionflag; 
        source1[i].qualityflag = source[i].qualityflag; 
        source1[i].rawcountrate = source[i].rawcountrate; 

    }
  

    F90MODPROC( addflags, addsourceflags )(&n, source1);

 

    delete [] source1;


}


/***************************************************************
*
*
****************************************************************/
/*
windowParameters getWindowParameters( string fileName )

{

 
    windowParameters parameters;    

    
   
    char name[256];
    strcpy( name, fileName.c_str());

    F90MODPROC(coordinates, windowparameters)( name, &parameters.x0, &parameters.y0, 
                  &parameters.xBin, &parameters.yBin);

    return parameters;

}
*/
/***************************************************************
*
*
****************************************************************/
windowParameters getWindowParameters( string fileName )

{

 
    windowParameters parameters;    
   
    char name[256];
    int stringLength1=fileName.length();
    //strcpy( name, fileName.c_str());

    sprintf(name, "%s", fileName.c_str());
    size_t stringLength2=strlen(name);

 
    unsigned char name3u[256];

    // passing strings to fortran see:
    // http://www.xgear.eu/callfortranfromc.html

    //cout << "L723 name="<< name << " stringLength1=" << stringLength1 << endl;
    //cout << "L729 stringLength2="<< stringLength2 << endl;
    

    for (int ii=0; ii<256; ii++){
      name3u[ii]='\0';
    }
    for (int ii=0;ii<stringLength1; ii++){
      name3u[ii]=(unsigned char)name[ii];
    }

    F90MODPROC(coordinates, windowparameters1)(&name3u[0], &stringLength2, &parameters.x0, 
			       &parameters.y0, &parameters.xBin, &parameters.yBin);
    return parameters;

}

/********************************************************************************
*
********************************************************************************/
void convertCoords( windowParameters &parameter, float &x, float &y)
{
    F90MODPROC( coordinates, convertcoords)( &x, &y, &parameter.x0, &parameter.y0, &parameter.xBin, &parameter.yBin);
}







/*The old flags
!****************************************************************************
! QFLAG source quality flags convention
! BIT NO             FLAGGING                         Integer
!
!   0                BAD PIXEL                                 1
!   1                READ-OUT STREAK                           2
!   2                SMOKE-RING                                4
!   3                SOURCE ON STAR-SPIKE                      8
!   4                Mod-8 source                             16
!   5                Source lies near to a bright source      32
!   6                Source within central enhancement        64
!*****************************************************************************
!****************************************************************************
! CFLAG source confusion flags convention
! BIT NO             FLAGGING                                           Integer
!
!   0                one or more source within 0.5-1.0 aperture radius  1 
!   1                one or more source within 0.5 aperture radius      2
!   2                point source lies with 1.0 aperture radius of      4
!                edge or some point of extended source lies on edge      
!   3           Point-source lies partly or entirely within extended    8
!                source region 
!*****************************************************************************


THe new ones


   "0) Source on a bad pixel                       (red)                                                   ",   &
   "1) Source on a read-out streak                 (magenta)                                               ",   &
   "2) Source near or within a smoke-ring          (yellow)                                                ",   &
   "3) Source on or near a diffraction-spike       (white)                                                 ",   &
   "4) Source surrounded by a strong mod-8 pattern (black)                                                 ",   & 
   "5) Source within central enhanced region       (red)                                                   ",   &
   "6) Source lies near to a bright source         (black)                                                 ",   &
   "7) Point source lies with 1.0 aperture radius of edge/some point of extended source lies on edge (blue)",   &
   "8) Point-source lies partly or entirely within extended source region (red)                            ",   &
   "9) Compact source/exceptionally strange        (red)                                                   "/)    








*/
/***************************************************************************
*
**************************************************************************/
void resetflags(  vector < OmSourceParameter > &source)
{

    for(unsigned i = 0; i < source.size(); i++)
    {
        int qflag = 0, cflag=0;

        if(source[i].qualityflag & (1 << 0))  // Bad pixel
	    qflag |= (1 << 0);

        if(source[i].qualityflag & (1 << 1))  // Read-out streak
	    qflag |= (1 << 1);



	/*********************************************************************
	 * NB SKIP SMOKE RINGS AND DIFFRACTION SPIKES - KNOWN TO BE UNRELIABLE
	 ************************************************************************/
	//        if(source[i].qualityflag & (1 << 2))  // Smoke ring
	//	  qflag |= (1 << 2);

	//        if(source[i].qualityflag & (1 << 3))  // Diffraction spike
	//	  qflag |= (1 << 3);

        if(source[i].qualityflag & (1 << 4))  // Mod-8 pattern
	    qflag |= (1 << 4);





	if(source[i].qualityflag & (1 << 6))  // Central enhancement
	    qflag |= (1 << 5);




	if(source[i].qualityflag & (1 << 5))  // Near a bright source
	    qflag |= (1 << 6);




        if( source[i].confusionflag & (1 << 2))  // near an edge
	{
            qflag |= (1 << 7);

        } 



        if( source[i].extensionflag == 0 && source[i].confusionflag & (1 << 3))  // 
	{
	    qflag |= (1 << 8);     // Point source within an extended-source region

        } 




        if( source[i].confusionflag & (1 << 0))  // 
	{
	    cflag |= (1 << 0);     

        } 





	//     source[i].qualityflag = qflag;
	//	continue;




        if( source[i].confusionflag & (1 << 1))  // 
	{
	    cflag |= (1 << 1);     

        } 

        source[i].qualityflag = qflag;

        source[i].confusionflag = cflag;

    }


}
/**************************************************************************
*
*
***************************************************************************/
void setConfusionFlags( vector < OmSourceParameter > &source)
{



    double maxRadius = double(12);
    double halfRadius = double(6);
  
 
    for(unsigned i = 0; i < source.size(); i++)
    {
        source[i].confusionflag = 0;
        if( source[i].extensionflag == 1)
        {
     
            continue;

        } 
        double maxRad = 10000.0;
        for(unsigned j = 0; j < source.size(); j++)
        {
	   if(i == j) continue;
      

           if( source[j].extensionflag == 1)
           {
	     
               continue;

           } 


           /********************************************************
	   * Compute the distance between the two sources
	   *********************************************************/

           double dx = (double)source[i].xcoord - (double)source[j].xcoord;
           double dy = (double)source[i].ycoord - (double)source[j].ycoord;

           maxRad = min( maxRad, hypot( dx, dy ) );
           
        }
        if( maxRad <= halfRadius) // Bit 1 should be set
	{

	    source[i].confusionflag |= (1 << 1);     
	     
        }
        else if( maxRad <= maxRadius) // Bit 0 should be set on sources i and j
	{
             
	    source[i].confusionflag |= (1 << 0);     
	  

       }


    }
}

