// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef POLINT
#define POLINT



template<class kind>
class Polint

{
private:
    int n;
    double *xa;
    double *ya;
    double *c;
    double *d;
    kind dy;
public:    
    void Initialise(int n, kind *x, kind *y);
    Polint() : n(0), xa(NULL), ya(NULL), c(NULL), d(NULL)  {};
    Polint(int size);
    Polint(int nIn, kind *xIn, kind *yIn);
    kind Interpolate(kind x);
    ~Polint();
};




/*****************************************************
 * Constructor
 ****************************************************/
template<class kind>
Polint<kind>::Polint(int nIn, kind *xIn, kind *yIn) 
  : n(0), xa(NULL), ya(NULL), c(NULL), d(NULL)
{
    Initialise(nIn, xIn, yIn);
   
}; 


/*****************************************************
 * Constructor
 ****************************************************/
template<class kind>
Polint<kind>::Polint(int size) 

{
    n = size;
 
    xa = new double[nIn];
    ya = new double[nIn];
    c = new double[nIn];
    d = new double[nIn];
     
 
}; 

/*****************************************************
 * Initialise
 ****************************************************/
template<class kind>
void Polint<kind>::Initialise(int nIn, kind *xIn, kind *yIn)
{

    if (nIn > n)
    
    {   
      
        if (xa) delete [] xa;
        if (ya) delete [] ya;
        if (c) delete [] c;
        if (d) delete [] d;

        xa = new double[nIn];
        ya = new double[nIn];
        c = new double[nIn];
        d = new double[nIn];
     
       

    }
    n = nIn;
    
    cout << "polint data\n";
    for (int i = 0; i < nIn; i++)
    { 
        xa[i] = (double)xIn[i];
        ya[i] = (double)yIn[i];
	//	cout << i << " x=" << xa[i] << " y=" << ya[i] << endl;      
    }
  
  
}; 
/***********************************************
 *
 **********************************************/
template<class kind>
Polint<kind>::~Polint()
{
    if (xa) delete [] xa;
    if (ya) delete [] ya;
    if (c) delete [] c;
    if (d) delete [] d;   
};    

/******************************************************
*
******************************************************/
template <class kind>
kind Polint<kind>::Interpolate(kind x)
{

    double den, dift, ho, hp, w; 
    double dif = fabs(x - xa[0]);
  
    int ns = 0;
   
    for (int i = 0; i < n; i++)
    {
        dift = fabs(x - xa[i]);
        if (dift < dif) 
        {
            ns = i;
            dif = dift;
        }
        c[i] = ya[i];    
        d[i] = ya[i];
    }
           
    double y = ya[ns--];
    for (int m = 1; m < (n - 1); m++) 
    {
      for (int i = 0; i < n - m; i++)
      {
          ho = xa[i] - x;
          hp = xa[i + m] - x;
          w = c[i + 1] - d[i];
          den = ho - hp;
          den = w / den;
          d[i] = hp * den;
          c[i] = ho * den;
       
      }

      y += (dy = (2 * ns < (n - m) ? c[ns + 1] : d[ns--]));
     
    }
  
    return y;
};











#endif
