// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      omsource.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Aug  9 17:05:36 MEST 2002
#include "sourcelistchecker.h"
#include <iostream>
#include "Dal.h"
#include <sstream>
#include<fstream>
#include <iomanip>
#include "sourcelist.h"
#include "OmImage.h"
#include "allocate.h"
#include "mode.h"
void ConvertCoords(float &xCoord, float& yCoord, int x0, int y0, int xBin, int yBin);
void checkImage(const string &, int size, const float* xpos, const float* ypos, const float *ignificance, const float *semia); 


bool badPixels(float **image, int nx, int ny, 
               const float x, const float y,
               int &i0, int &i0, float& maxPixel);



void checkImageStatistics( float **image, int nx, int ny, string filter, float exposure);



void sourcelistchecker()
{
  


    string fileName = stringParameter( "set");
   

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);




    string filter = dataSet->attribute( "FILTER" )->asString();   

    if ( (filter.find("GRI", 0) != string::npos) || (filter.find("Gri", 0) != string::npos) ||  (filter.find("gri", 0) != string::npos))
    {

       ofstream output("grism.dat", ios::app);
       char buffer[100];
       sprintf(buffer, "%40s%10s", fileName.c_str(), filter.c_str());
       output << buffer <<  endl;



       output.close();

       dataSetServer->close(dataSet); // Finished with the file


       exit(0);;

    }

   







    cout << "Checking sourcelist file " << endl;   
    

    // Read the table
    const Table *table = dataSet->table( "SRCLIST" );

    int size = table->rows();

 


    float *xpos = LoadColumnData( table, "XPOS");
    if ( xpos == 0)
    {

        dataSetServer->close(dataSet); // Finished with the file


        Sas::fatal("No XPOS column", fileName);
    }	    




    float *ypos = LoadColumnData( table, "YPOS");
    if ( xpos == 0)
    {

        dataSetServer->close(dataSet); // Finished with the file


        Sas::fatal("No YPOS column", fileName);
    }	    





    float *significance = LoadColumnData( table, "SIGNIFICANCE");
    if ( xpos == 0)
    {

        dataSetServer->close(dataSet); // Finished with the file


        Sas::warning("No SIGNIFICANCE column", fileName);
    }	    







    float *semiA = LoadColumnData( table, "FWHM_MAJ");

    if ( xpos == 0)
    {

        dataSetServer->close(dataSet); // Finished with the file

         
        Sas::warning("No FWHM_MAJ column", fileName);
        exit(0);
    }	    









 
 

    bool checked = false;
    for(int i = 0; i < size; i++)
    {    
        if (significance[i] >= 10.0 && semiA[i] <= 0.6)
	{    

	
	  checkImage(fileName, size, xpos, ypos, significance, semiA);
          checked = true; 
            break;

       }



    }
    
    if (!checked)
        checkImage(fileName, 0, xpos, ypos, significance, semiA);
    
 




    if (significance) delete [] significance;
    if (semiA) delete [] semiA;
    if (xpos) delete [] xpos;
    if (ypos) delete [] ypos;

 


};

/************************************************************
*
*
*************************************************************/
void ConvertCoords(float &xCoord, float& yCoord, int x0, int y0, int xBin, int yBin)
{
    


  xCoord = xCoord * float(xBin) + float(x0);
  yCoord = yCoord * float(yBin) + float(y0);

}

/*************************************************************
 *
 *************************************************************/
void checkImage(const string &srcListFile, int size, const float* xpos, const float* ypos, 
                          const float *significance, const float *semiA) 
{     



    int nx = 0, ny = 0;


    string imageFile = stringParameter("imageset");



    DataSet *dataSet = dataSetServer->open(imageFile, Dal::Read);


   string filter = dataSet->attribute( "FILTER" )->asString();   




        

    float exposure = dataSet->attribute("EXPOSURE")->asReal();


    int xBin = dataSet->attribute("BINAX1")->asInt();
    int yBin = dataSet->attribute("BINAX2")->asInt();


    bool binBPE = dataSet->attribute( "BINBPE" )->asBool();


    xBin = (int)pow(2.0, xBin);
    yBin = (int)pow(2.0, yBin);


    if (binBPE) 
    {
        xBin *= 2;
        yBin *= 2;
    }



    string windowString = dataSet->attribute("DETSEC")->asString();







    dataSetServer->close(dataSet); // Finished with the file





 
    istringstream text(windowString);

    int windowX0, windowX1, windowY0, windowY1;

    char c;
    text >> c >> windowX0 >> c >> windowX1 >> c >> windowY0 >> c >> windowY1 >> c;
 

    windowX0 *= xBin;
    windowY0 *= yBin;






    float** image = LoadImage( (char *)imageFile.c_str(), nx, ny);

    string outputfile = "badsources_" + filter + ".dat";
    ofstream output(outputfile.c_str(), ios::app);



    for(int i = 0; i < size; i++)
    {    
        if (significance[i] >= 10.0 && semiA[i] <= 0.6)
	{    

	

          float x = xpos[i], y = ypos[i]; 


          int i0 = 0, j0 = 0;
          float maxPixel = 0.0;  
          if (badPixels(image, nx, ny, x, y, i0, j0, maxPixel) )
	  {



	      cout << i << ") x=" << xpos[i] << " y=" << ypos[i] << " significance=" 
		   << significance[i] << " semiA=" << semiA[i] <<
		" i0=" << i0 << " j0=" << j0 << " pixel=" << maxPixel << endl; 
  


              ConvertCoords(x, y, windowX0, windowY0, xBin, yBin);
              char buffer[200];
              sprintf( buffer, "%40s   %8d) %10.2f %10.2f %8.1f %7.2f %8.1f %7d %7d %10.1f %6d %6d", 
                       srcListFile.c_str(), (i + 1), x, y, significance[i], semiA[i], 
                       exposure, i0, j0, maxPixel, nx, ny); 
              output << buffer << endl;
          } 

       }


    }

    for (int i = 0; i < nx; i++)
    {
        for (int j = 0;j < ny; j++)
	{

            float maxPixel = 0.0;  
            int i0 = i;
            int j0 = j;
            float x = (float)(i + 1);
            float y = (float)(j + 1);
            if ( image[i][j] < 60000.0 ) continue;
            if (badPixels(image, nx, ny, x, y, i0, j0, maxPixel) )
	    {



              ConvertCoords(x, y, windowX0, windowY0, xBin, yBin);
              char buffer[200];
              sprintf( buffer, "%40s   %8d) %10.2f %10.2f %8.1f %7.2f %8.1f %7d %7d %10.1f %6d %6d", 
                       srcListFile.c_str(), -1, x, y, -1.0, -1.0, 
                       exposure, i0, j0, maxPixel, nx, ny); 
              output << buffer << endl;



            }

        } 
            


    }




    output.close();








    checkImageStatistics( image, nx, ny, filter, exposure);


    if (image) de_allocate(&image, nx);

}
/**************************************************
*
*
**************************************************/
bool badPixels(float **image, int nx, int ny, 
               const float x, const float y,
               int &i0, int &j0, float& maxPixel)
{
    i0 = int(x - 0.5);
    j0 = int(y - 0.5);


    i0 = max(0, i0);
    j0 = max(0, j0);


    i0 = min(nx - 1, i0);
    j0 = min(ny - 1, j0);


    int dx = 10;

    int il = i0 - dx;
    int jl = j0 - dx;


    il = max(0, il);
    jl = max(0, jl);


    il = min(nx - 1, il);
    jl = min(ny - 1, jl);


   



    int iu = i0 + dx;
    int ju = j0 + dx;




    iu = min(nx - 1, iu);
    ju = min(ny - 1, ju);


    maxPixel = 0.0;
    double sum1 = double(0), sum2 = double(0);
    int count1 = 0, count2 = 0; 
    float maxImage = 0.0;    
    for (int i = il; i <= iu; i++)
    {
        for (int j = jl; j <= ju; j++)
        {
	  if ( image[i][j] > 65536.0) 
	  {

	      if (image[i][j] > maxPixel)
	      {
		  maxPixel = image[i][j];
                  i0 = i;
                  j0 = j; 

              }            

	      count1++;
	      //              sum1 += (double)image[i][j]; 
	      image[i][j] = 0.0;

          }
          else
	  {
	      maxImage = max(maxImage, image[i][j]);
              count2++;
	      //	      sum2 += (double)image[i][j]; 


          }


        }

    }

    if ( count1 == 0) return false;
    if ( count1 > 2 ) return false;
    if ( maxImage > 5000.0) return false;
  

    return true;

    sum2 /= (double)count2;
    sum2 = max(sum2, (double)1);
    sum1 /= (double)count1;

  


    if ( (sum1 / sum2) > 20.0 )
	return true;
    else
	return false;


}
/************************************************************
*
*
************************************************************/
void checkImageStatistics( float **image, int nx, int ny, string filter, float exposure)
{



  return;

    float mean = 0.0, median = 0.0;


 
    vector<float> data;


    for (int i = 0; i < nx; i++)
      for (int j = 0; j < ny; j++)
	data.push_back( image[i][j] );

    float mode = halfRangeMode(data, mean, median);



    string imageFile = stringParameter("imageset");




    string outputfile = "imagefiles_" + filter + ".dat";
    ofstream output(outputfile.c_str(), ios::app);



     char buffer[200];
     sprintf( buffer, "%40s   %6d %6d %8.1f %6.1f %6.1f %6.1f", 
	      imageFile.c_str(), nx, ny, exposure, mean, median, mode); 
                       

     output << buffer << endl;
    




    output.close();








}
